__all__ = [
    'b94encode', 'b94decode', 'rs_b94encode', 'rs_b94decode', 'py_b94encode',
    'py_b94decode', 'b72encode', 'b72decode', 'rs_b72encode', 'rs_b72decode',
    'py_b72encode', 'py_b72decode'
]

from .base94 import b94encode as py_b94encode, b94decode as py_b94decode

try:
	from ._base94 import b94encode, b94decode  # type: ignore
	rs_b94encode, rs_b94decode = b94encode, b94decode
except ImportError:
	rs_b94encode, rs_b94decode = None, None

b94encode = py_b94encode if rs_b94encode is None else rs_b94encode

b94decode = py_b94decode if rs_b94decode is None else rs_b94decode

from .base72 import b72encode as py_b72encode, b72decode as py_b72decode

try:
	from ._base94 import b72encode, b72decode  # type: ignore
	rs_b72encode, rs_b72decode = b72encode, b72decode
except ImportError:
	rs_b72encode, rs_b72decode = None, None

b72encode = py_b72encode if rs_b72encode is None else rs_b72encode

b72decode = py_b72decode if rs_b72decode is None else rs_b72decode



# Implementation of Base94 encode and decode in Python 3.
#
# THE GPLv3 LICENSE
# Copyleft (©) 2022 hibays
#

''' 
For example:

>>> from base94 import *

>>> b94encode(b'ABC')
b'Wg\\`y'

>>> b94encode(b'123456')
b'Q<-{{@fN'

>>> b94decode(b'Wg\\`y')
b'ABC'

>>> b94decode(b'Q<-{{@fN')
b'123456'
'''

__all__ = ['b94encode', 'b94decode']

_b94alphabet = (b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~')
#_b94alphabet = sorted(_b94alphabet)

def b94encode(data,
	## HACK: turn globals into locals
	len=len,
	range=range,
	bnjoin=b''.join,
	from_bytes=int.from_bytes,
	b94tab=[bytes((i,)) for i in _b94alphabet] # The type of (b'ABC')[0] is int. This makes it from int to bytes.
	) -> bytes :
	'''Input bytes-like object, return bytes.
	This algorithm transform 9 bytes to 11 bytes
	The encoded data is about 22.2% larger.
	'''
	b94tab2 = [bnjoin((i, j)) for i in b94tab for j in b94tab] # A simple magic.

	datlen = len(data)
	padding = (-datlen) % 9
	if padding :
		data, datlen = \
			data + b'\0' * padding, datlen + padding
	
	encoded = []
	for i in range(0, datlen, 9) :
		c = from_bytes(data[i: i + 9], 'big') # from data get 9 bytes data to int
		
		# This is faster.
		d2_3 = c // 94
		d4_5 = d2_3 // 8836
		d6_7 = d4_5 // 8836
		d8_9 = d6_7 // 8836

		encoded.append(bnjoin(
			(
				b94tab2[d8_9 // 8836],
				b94tab2[d8_9 % 8836],
				b94tab2[d6_7 % 8836],
				b94tab2[d4_5 % 8836],
				b94tab2[d2_3 % 8836],
				b94tab[c % 94]
			)
		))

	if padding :
		encoded[-1] = encoded[-1][: -padding]

	return bnjoin(encoded)
	
def b94decode(data,
	## HACK: turn globals into locals
	len=len,
	range=range,
	bnjoin = b''.join,
	_intto_byte = int.to_bytes,
	b94nums = {j: i for i,j in enumerate(_b94alphabet)} # dict faster
	) -> bytes :
	'''This function decodes the data that has been encoded by base94.
	Input bytes-like object, return bytes.
	'''
	datlen = len(data)
	padding = (-datlen) % 11
	if padding :
		data, datlen = \
			data + b'~' * padding, datlen + padding
	
	# To revert base94-encoded data. 11 bytes -> 9 bytes.
	# Just like a number of base 94 convert to decimal, each chunk is a number.
	result = bnjoin(
		_intto_byte((((((((((
				 b94nums[data[i    ]] # byte 11
		  * 94 + b94nums[data[i + 1]] # byte 10
		) * 94 + b94nums[data[i + 2]] # byte 9
		) * 94 + b94nums[data[i + 3]] # byte 8
		) * 94 + b94nums[data[i + 4]] # byte 7
		) * 94 + b94nums[data[i + 5]] # byte 6
		) * 94 + b94nums[data[i + 6]] # byte 5
		) * 94 + b94nums[data[i + 7]] # byte 4
		) * 94 + b94nums[data[i + 8]] # byte 3
		) * 94 + b94nums[data[i + 9]] # byte 2
		) * 94 + b94nums[data[i +10]] # byte 1
		, 9, 'big')
		 for i in range(0, datlen, 11)
	)
	
	if padding :
		result = result[: -padding]

	return result

if __name__ == '__main__' :
	from os import system as osSys, name as osName
	osSys('cls' if osName in ('nt', 'dos') else 'clear')

	def c(size, depth=0, pint=('b', 'Kb', 'Mb', 'Gb', 'Tb', 'Pb'))-> str :
		if size > 1024 :
			return c(size/1024, depth+1)
		return '%f%s' % (size, pint[depth])
		
	try :
		from base91 import encode as b91encode, decode as b91decode # type: ignore
	except :
		b91decode = b91encode = lambda _ : str(print('(No Base91 Modular)', end=''))

	from base64 import b64encode, b64decode, b85encode, b85decode
	
	from time import perf_counter
	from os import urandom as osUrandom
	data = osUrandom(1048575)#hash(perf_counter()) & 1048575*16//8)
	
	def _test_mes(base, enData, norData=data) :
		'''A function uses to test the encode and decode function.'''
		t = perf_counter()
		deData = eval('b%ddecode' % base)(enData)
		t = perf_counter() - t
		print('base%d done decoded in %f seconds.' % (base, t))
		if deData == norData :
			return 'Succese! Base%s-encoded data can 100%% revert to normal data.' % (base)
		from difflib import SequenceMatcher as dataSimilarity
		bdesim = dataSimilarity(None, deData, norData).quick_ratio()
		return 'Bad new! Cannot revert %f%% of base%d-encoded data.' % (1e2 - 1e2*bdesim, base)

	def _test_en(base, norData=data) :
		t = perf_counter()
		enTest = eval('b%dencode'%base)(norData)
		t = perf_counter() - t
		print('base%d done encoded in %f seconds.' % (base, t))
		return enTest

	en64,en85,en91,en94 = _test_en(64),_test_en(85),_test_en(91),_test_en(94)
	lno,l64,l85,l91,l94 = len(data), len(en64), len(en85), len(en91), len(en94)
	la = lambda n : 100*n / lno - 100
	
	print()
	#print('normal: %s\nbase64: %s\nbase85: %s\nbase91: %s\nbase94: %s\n'%(data, en64, en85, en91, en94))
	print('normal: %s\nbase64: %d(%.2f%%)\nbase85: %d(%.2f%%)\nbase91: %d(%.2f%%)\nbase94: %d(%.2f%%)\n'%(c(lno), l64, la(l64), l85, la(l85), l91, la(l91), l94, la(l94)))
	print(          '\nbase64: %s\nbase85: %s\nbase91: %s\nbase94: %s\n'%(_test_mes(64, en64), _test_mes(85, en85), _test_mes(91, en91), _test_mes(94, en94)))



import timeit
import os
import sys
from functools import partial

# 测试数据大小配置（单位：字节）
TEST_CASES = [
    ("10KB", 10*1024),
    ("100KB", 100*1024),
    ("1MB", 1024*1024),
    ("10MB", 10*1024*1024),
]

def verify_correctness(impl):
    """验证实现正确性"""
    test_data = b"Base94 Test String"
    try:
        encoded = impl['encode'](test_data)
        decoded = impl['decode'](encoded)
        assert decoded == test_data, "Decode result mismatch"
    except Exception as e:
        print(f"⚠️ {impl['name']} 实现验证失败: {str(e)}")
        sys.exit(1)

def run_benchmark(impl, data):
    """执行单个基准测试"""
    results = {'encode': [], 'decode': []}
    
    # 预热缓存
    impl['encode'](data[:1024])
    impl['decode'](impl['encode'](data[:1024]))
    
    # 编码测试
    encode_timer = timeit.Timer(
        partial(impl['encode'], data),
        setup='gc.enable()'
    )
    results['encode'] = encode_timer.repeat(number=10, repeat=3)
    
    # 解码测试
    encoded_data = impl['encode'](data)
    decode_timer = timeit.Timer(
        partial(impl['decode'], encoded_data),
        setup='gc.enable()'
    )
    results['decode'] = decode_timer.repeat(number=10, repeat=3)
    
    return {
        'encode': min(results['encode']),
        'decode': min(results['decode'])
    }

def format_speed(bytes_size, seconds):
    """计算并格式化速度"""
    if seconds == 0:
        return "∞ MB/s"
    mb = bytes_size / (1024*1024)
    return f"{mb/seconds:.2f} MB/s"

def main():
    try:
        # 导入不同实现
        import base94
        
        implementations = [
            {
                'name': 'Python Native',
                'encode': base94.py_b94encode,
                'decode': base94.py_b94decode
            },
            {
                'name': 'Rust Accelerated',
                'encode': base94.rs_b94encode,
                'decode': base94.rs_b94decode
            }
        ]
        
        # 预先验证所有实现
        for impl in implementations:
            verify_correctness(impl)
        
        print("🚀 开始性能基准测试...\n")
        
        # 执行测试
        results = {}
        for case_name, data_size in TEST_CASES:
            print(f"\n🔧 生成测试数据: {case_name}...")
            data = os.urandom(data_size)
            results[case_name] = []
            
            for impl in implementations:
                print(f"🔍 测试 {impl['name']} ({case_name})...")
                try:
                    stats = run_benchmark(impl, data)
                    results[case_name].append({
                        'name': impl['name'],
                        'encode_time': stats['encode'],
                        'decode_time': stats['decode'],
                        'encode_speed': format_speed(data_size, stats['encode']),
                        'decode_speed': format_speed(data_size, stats['decode'])
                    })
                except MemoryError:
                    print(f"❌ 内存不足，跳过 {case_name} 测试")
                    continue
        
        # 打印结果表格
        print("\n📊 测试结果汇总:")
        print("| 数据大小 | 实现版本         | 编码时间 (s) | 解码时间 (s) | 编码速度   | 解码速度   |")
        print("|----------|------------------|--------------|--------------|------------|------------|")
        for case in TEST_CASES:
            case_name = case[0]
            for result in results.get(case_name, []):
                print(f"| {case_name:8} | {result['name']:16} | "
                      f"{result['encode_time']:12.4f} | {result['decode_time']:12.4f} | "
                      f"{result['encode_speed']:10} | {result['decode_speed']:10} |")
        
        # 打印性能提升比例
        print("\n💹 性能提升比例:")
        for case in TEST_CASES:
            case_name = case[0]
            if len(results.get(case_name, [])) == 2:
                py = results[case_name][0]
                rs = results[case_name][1]
                encode_ratio = py['encode_time'] / rs['encode_time']
                decode_ratio = py['decode_time'] / rs['decode_time']
                print(f"{case_name}:")
                print(f"  编码速度提升: {encode_ratio:.1f}x")
                print(f"  解码速度提升: {decode_ratio:.1f}x")
        
    except ImportError as e:
        print(f"❌ 导入错误: {str(e)}")
        print("请确保已正确安装：")
        print("1. Python原生版本: pip install base94")
        print("2. Rust加速版本: maturin develop --release")
        sys.exit(1)

if __name__ == "__main__":
    main()


from os import urandom
from base94 import b94decode
from base94 import py_b94decode
from base94 import rs_b94decode

def test_b94decode_default():
    """
    Test base94 encoding.
    """
    assert b94decode(b'') == b''
    assert b94decode(b'Wg\\`y') == b'ABC'
    assert b94decode(b'Q<-{{@fN') == b'123456'

def test_b94decode_py():
    """
    Test base94 encoding.
    """
    assert py_b94decode(b'') == b''
    assert py_b94decode(b'Wg\\`y') == b'ABC'
    assert py_b94decode(b'Q<-{{@fN') == b'123456'

def test_b94decode_rs():
    """
    Test base94 encoding.
    """
    assert rs_b94decode(b'') == b''
    assert rs_b94decode(b'Wg\\`y') == b'ABC'
    assert rs_b94decode(b'Q<-{{@fN') == b'123456'


from os import urandom
from base94 import b94encode
from base94 import py_b94encode
from base94 import rs_b94encode

def test_b94encode_default():
    """
    Test base94 encoding.
    """
    assert b94encode(b'') == b''
    assert b94encode(b'ABC') == b'Wg\\`y'
    assert b94encode(b'123456') == b'Q<-{{@fN'

def test_b94encode_py():
    """
    Test base94 encoding.
    """
    assert py_b94encode(b'') == b''
    assert py_b94encode(b'ABC') == b'Wg\\`y'
    assert py_b94encode(b'123456') == b'Q<-{{@fN'

def test_b94encode_rs():
    """
    Test base94 encoding.
    """
    assert rs_b94encode(b'') == b''
    assert rs_b94encode(b'ABC') == b'Wg\\`y'
    assert rs_b94encode(b'123456') == b'Q<-{{@fN'


# base72.py
# Implementation of Base72 encode and decode in Python 3.
#
# THE GPLv3 LICENSE
# Copyleft (©) 2025 hibays
#
'''
For example:

>>> from base72 import *

>>> b72encode(b'ABC')
b'K7g'

>>> b72encode(b'1234567890')
b'K7g0BpX5K7g'

>>> b72decode(b'K7g0BpX5K7g')
b'1234567890'
'''

__all__ = ['b72encode', 'b72decode']

_b72alphabet = b'0123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ_-+=()[]{}@,;'


def b72encode(
    data,
    len=len,
    range=range,
    bnjoin=b''.join,
    from_bytes=int.from_bytes,
    b72tab=tuple(bytes((i, )) for i in _b72alphabet)  # int → bytes
) -> bytes:
	'''Input bytes-like object, return bytes.
	This algorithm transforms 10 bytes to 13 Base72 characters.
	The encoded data is about 30% larger.
	'''
	# Precompute double-char table for performance
	b72tab2 = [bnjoin((i, j)) for i in b72tab for j in b72tab]

	datlen = len(data)
	padding = (-datlen) % 10  # bytes to pad to multiple of 10
	if padding:
		data, datlen = data + b'\0' * padding, datlen + padding

	encoded = []
	for i in range(0, datlen, 10):
		# Convert 10 bytes to big-endian integer (treat as 80-bit number)
		c = from_bytes(data[i:i + 10], 'big')

		# This is faster.
		d2_3 = c // 72
		d4_5 = d2_3 // 5184
		d6_7 = d4_5 // 5184
		d8_9 = d6_7 // 5184
		d10_11 = d8_9 // 5184

		encoded.append(bnjoin(
			(
				b72tab2[d10_11 // 5184],
				b72tab2[d10_11 % 5184],
				b72tab2[d8_9 % 5184],
				b72tab2[d6_7 % 5184],
				b72tab2[d4_5 % 5184],
				b72tab2[d2_3 % 5184],
				b72tab[c % 72]
			)
		))

	if padding:
		encoded[-1] = encoded[-1][:-padding]

	return bnjoin(encoded)


def b72decode(
    data,
    len=len,
    range=range,
    bnjoin=b''.join,
    to_bytes=int.to_bytes,
    b72nums={j: i for i,j in enumerate(_b72alphabet)}
) -> bytes:
	'''This function decodes the data that has been encoded by base72.
	Input bytes-like object, return bytes.
	'''
	datlen = len(data)
	padding = (-datlen) % 13  # chars to pad to multiple of 13
	if padding:
		data, datlen = data + b';' * padding, datlen + padding

	# To revert base72-encoded data. 13 bytes -> 10 bytes.
	# Just like a number of base 72 convert to decimal, each chunk is a number.
	result = bnjoin(
		to_bytes((((((((((((
				 b72nums[data[i    ]] # byte 13
		  * 72 + b72nums[data[i + 1]] # byte 12
		) * 72 + b72nums[data[i + 2]] # byte 11
		) * 72 + b72nums[data[i + 3]] # byte 10
		) * 72 + b72nums[data[i + 4]] # byte 9
		) * 72 + b72nums[data[i + 5]] # byte 8
		) * 72 + b72nums[data[i + 6]] # byte 7
		) * 72 + b72nums[data[i + 7]] # byte 6
		) * 72 + b72nums[data[i + 8]] # byte 5
		) * 72 + b72nums[data[i + 9]] # byte 4
		) * 72 + b72nums[data[i +10]] # byte 3
		) * 72 + b72nums[data[i +11]] # byte 2
		) * 72 + b72nums[data[i +12]] # byte 1
		, 10, 'big')
		 for i in range(0, datlen, 13)
	)

	if padding:
		result = result[:-padding]

	return result


if __name__ == '__main__':
	from os import system as osSys, name as osName
	osSys('cls' if osName in ('nt', 'dos') else 'clear')

	def c(size, depth=0, pint=('b', 'Kb', 'Mb', 'Gb', 'Tb', 'Pb')) -> str:
		if size > 1024:
			return c(size / 1024, depth + 1)
		return '%f%s' % (size, pint[depth])

	try:
		from base91 import encode as b91encode, decode as b91decode  # type: ignore
	except ImportError:
		b91encode = b91decode = lambda _: print(
		    '(No Base91 Module)', end=''
		) or b''

	from base64 import b64encode, b64decode, b85encode, b85decode
	from time import perf_counter
	from os import urandom

	data = urandom(1048575)  # 1MB random data

	def _test_mes(base, enData, norData=data):
		'''Test decode correctness and speed.'''
		t = perf_counter()
		deData = eval(f'b{base}decode')(enData)
		t = perf_counter() - t
		print(f'base{base} decoded in {t:.6f} seconds.')
		if deData == norData:
			return f'Success! Base{base}-encoded data reverted 100%.'
		from difflib import SequenceMatcher
		sim = SequenceMatcher(None, deData, norData).quick_ratio()
		return f'Failed! Only {sim*100:.9f}% match.'

	def _test_en(base, norData=data):
		t = perf_counter()
		enTest = eval(f'b{base}encode')(norData)
		t = perf_counter() - t
		print(f'base{base} encoded in {t:.6f} seconds.')
		return enTest

	# Test encoding
	en64 = _test_en(64)
	en85 = _test_en(85)
	en72 = _test_en(72)
	en91 = _test_en(91) if 'b91encode' in globals() else b''
	en94 = b''  # 如果你有 base94.py，可以加上

	lno = len(data)
	l64, l85, l72, l91 = len(en64), len(en85), len(en72), len(en91)
	la = lambda n: 100 * n / lno - 100

	print()
	print(f'normal: {c(lno)}')
	print(f'base64: {l64} ({la(l64):.2f}%)')
	print(f'base85: {l85} ({la(l85):.2f}%)')
	print(f'base72: {l72} ({la(l72):.2f}%)')
	if l91:
		print(f'base91: {l91} ({la(l91):.2f}%)')

	print()
	print(f'base64: {_test_mes(64, en64)}')
	print(f'base85: {_test_mes(85, en85)}')
	print(f'base72: {_test_mes(72, en72)}')
	if l91:
		print(f'base91: {_test_mes(91, en91)}')
